@@ -74,6 +74,13 @@ public:
 class DictionaryTag: public Tag
 {
 public:
+	/*
+	   Pointer to the class we are binded to
+
+	   This must ONLY be used when creating instances of the tag
+           There may be more than a class binded to the same tag. Only
+	   the first one is reported here. This behaviour is tested.
+	*/
 	Class_base* bindedTo;
 	RootMovieClip* loadedFrom;
 	DictionaryTag(RECORDHEADER h):Tag(h),bindedTo(NULL),loadedFrom(NULL){ }
@@ -298,8 +305,8 @@ protected:
 	class list_orderer
 	{
 	public:
-		bool operator()(const std::pair<PlaceInfo, DisplayObject*>& a, int d);
-		bool operator()(int d, const std::pair<PlaceInfo, DisplayObject*>& a);
+		bool operator()(const std::pair<PlaceInfo, DisplayObject*>& a, uint32_t d);
+		bool operator()(uint32_t d, const std::pair<PlaceInfo, DisplayObject*>& a);
 		bool operator()(const std::pair<PlaceInfo, DisplayObject*>& a, const std::pair<PlaceInfo, DisplayObject*>& b);
 	};
 
@@ -319,6 +326,8 @@ protected:
 	UI16_SWF ClipDepth;
 	CLIPACTIONS ClipActions;
 	PlaceObject2Tag(RECORDHEADER h):DisplayListTag(h){}
+	void assignObjectToList(DisplayObject* obj, MovieClip* parent,
+			std::list<std::pair<PlaceInfo, DisplayObject*> >::iterator listIterator) const;
 
 public:
 	STRING Name;
@@ -341,7 +350,6 @@ private:
 
 public:
 	PlaceObject3Tag(RECORDHEADER h, std::istream& in);
-	void execute(MovieClip* parent, std::list < std::pair<PlaceInfo, DisplayObject*> >& list);
 };
 
 class FrameLabelTag: public Tag
