@@ -75,15 +75,6 @@ void DoABCTag::execute(RootMovieClip*)
 {
 	LOG(LOG_CALLS,_("ABC Exec"));
 	sys->currentVm->addEvent(NULL,new ABCContextInitEvent(context));
-	SynchronizationEvent* se=new SynchronizationEvent;
-	bool added=sys->currentVm->addEvent(NULL,se);
-	if(!added)
-	{
-		se->decRef();
-		throw RunTimeException("Could not add event");
-	}
-	se->wait();
-	se->decRef();
 }
 
 DoABCDefineTag::DoABCDefineTag(RECORDHEADER h, std::istream& in):ControlTag(h)
@@ -112,15 +103,6 @@ void DoABCDefineTag::execute(RootMovieClip*)
 {
 	LOG(LOG_CALLS,_("ABC Exec ") << Name);
 	sys->currentVm->addEvent(NULL,new ABCContextInitEvent(context));
-	SynchronizationEvent* se=new SynchronizationEvent;
-	bool added=sys->currentVm->addEvent(NULL,se);
-	if(!added)
-	{
-		se->decRef();
-		throw RunTimeException("Could not add event");
-	}
-	se->wait();
-	se->decRef();
 }
 
 SymbolClassTag::SymbolClassTag(RECORDHEADER h, istream& in):ControlTag(h)
@@ -187,6 +169,7 @@ void ABCVm::registerClasses()
 	builtin->setVariableByQName("Function","",Class<IFunction>::getClass());
 	builtin->setVariableByQName("undefined","",new Undefined);
 	builtin->setVariableByQName("Math","",Class<Math>::getClass());
+	builtin->setVariableByQName("Namespace","",Class<Namespace>::getClass());
 	builtin->setVariableByQName("Date","",Class<Date>::getClass());
 	builtin->setVariableByQName("RegExp","",Class<RegExp>::getClass());
 	builtin->setVariableByQName("QName","",Class<ASQName>::getClass());
@@ -233,6 +216,7 @@ void ABCVm::registerClasses()
 	builtin->setVariableByQName("Stage","flash.display",Class<Stage>::getClass());
 	builtin->setVariableByQName("Graphics","flash.display",Class<Graphics>::getClass());
 	builtin->setVariableByQName("GradientType","flash.display",Class<GradientType>::getClass());
+	builtin->setVariableByQName("BlendMode","flash.display",Class<BlendMode>::getClass());
 	builtin->setVariableByQName("LineScaleMode","flash.display",Class<LineScaleMode>::getClass());
 	builtin->setVariableByQName("StageScaleMode","flash.display",Class<StageScaleMode>::getClass());
 	builtin->setVariableByQName("StageAlign","flash.display",Class<StageAlign>::getClass());
@@ -254,8 +238,10 @@ void ABCVm::registerClasses()
 	builtin->setVariableByQName("Font","flash.text",Class<Font>::getClass());
 	builtin->setVariableByQName("StyleSheet","flash.text",Class<StyleSheet>::getClass());
 	builtin->setVariableByQName("TextField","flash.text",Class<TextField>::getClass());
-	builtin->setVariableByQName("TextFieldType","flash.text",Class<ASObject>::getClass(QName("TextFieldType","flash.text")));
-	builtin->setVariableByQName("TextFormat","flash.text",Class<ASObject>::getClass(QName("TextFormat","flash.text")));
+	builtin->setVariableByQName("TextFieldType","flash.text",Class<TextFieldType>::getClass());
+	builtin->setVariableByQName("TextFieldAutoSize","flash.text",Class<TextFieldAutoSize>::getClass());
+	builtin->setVariableByQName("TextFormat","flash.text",Class<TextFormat>::getClass());
+	builtin->setVariableByQName("TextFormatAlign","flash.text",Class<TextFormatAlign>::getClass());
 
 	builtin->setVariableByQName("XMLDocument","flash.xml",Class<XMLDocument>::getClass());
 	builtin->setVariableByQName("XMLNode","flash.xml",Class<XMLNode>::getClass());
@@ -281,6 +267,8 @@ void ABCVm::registerClasses()
 	builtin->setVariableByQName("Matrix","flash.geom",Class<Matrix>::getClass());
 	builtin->setVariableByQName("Transform","flash.geom",Class<Transform>::getClass());
 	builtin->setVariableByQName("Point","flash.geom",Class<Point>::getClass());
+	builtin->setVariableByQName("Vector3D","flash.geom",Class<Vector3D>::getClass());
+	builtin->setVariableByQName("Matrix3D","flash.geom",Class<ASObject>::getClass(QName("Matrix3D", "flash.geom")));
 
 	builtin->setVariableByQName("EventDispatcher","flash.events",Class<EventDispatcher>::getClass());
 	builtin->setVariableByQName("Event","flash.events",Class<Event>::getClass());
@@ -711,15 +699,15 @@ multiname* ABCContext::getMultiname(unsigned int n, call_context* th)
 	{
 		m->cached=new multiname;
 		ret=m->cached;
-		ret->isAttribute=m->isAttributeName();
-
 		if(n==0)
 		{
 			ret->name_s="any";
 			ret->name_type=multiname::NAME_STRING;
 			ret->ns.emplace_back("",NAMESPACE);
+			ret->isAttribute=false;
 			return ret;
 		}
+		ret->isAttribute=m->isAttributeName();
 		switch(m->kind)
 		{
 			case 0x07:
@@ -1008,47 +996,44 @@ ABCContext::ABCContext(istream& in)
 		else
 			methods[method_body[i].method].body=&method_body[i];
 	}
-}
 
-ABCContext::~ABCContext()
-{
 #ifdef PROFILING_SUPPORT
-	if(sys->getProfilingOutput().len())
-	{
-		ofstream f(sys->getProfilingOutput().raw_buf());
-		f << "events: Time" << endl;
-		for(uint32_t i=0;i<methods.size();i++)
-		{
-			if(methods[i].profTime)
-			{
-				if(methods[i].name)
-				{
-					const multiname* m=getMultiname(methods[i].name,NULL);
-					f << "fn=" << *m << endl;
-				}
-				else
-					f << "fn=" << &methods[i] << " " << i << endl;
-				f << "1 " << methods[i].profTime << endl;
-				auto it=methods[i].profCalls.begin();
-				for(;it!=methods[i].profCalls.end();it++)
-				{
-					if(it->first->name)
-					{
-						const multiname* m=getMultiname(it->first->name,NULL);
-						f << "cfn=" << *m << endl;
-					}
-					else
-						f << "cfn=" << it->first << endl;
-					f << "calls=1 1" << endl;
-					f << "1 " << it->second << endl;
-				}
-			}
-		}
-		f.close();
-	}
+	sys->contextes.push_back(this);
 #endif
 }
 
+#ifdef PROFILING_SUPPORT
+void ABCContext::dumpProfilingData(ostream& f) const
+{
+	for(uint32_t i=0;i<methods.size();i++)
+	{
+		if(!methods[i].profTime.empty()) //The function was executed at least once
+		{
+			if(methods[i].validProfName)
+				f << "fn=" << methods[i].profName << endl;
+			else
+				f << "fn=" << &methods[i] << endl;
+			for(uint32_t j=0;j<methods[i].profTime.size();j++)
+			{
+				//Only output instructions that have been actually executed
+				if(methods[i].profTime[j]!=0)
+					f << j << ' ' << methods[i].profTime[j] << endl;
+			}
+			auto it=methods[i].profCalls.begin();
+			for(;it!=methods[i].profCalls.end();it++)
+			{
+				if(it->first->validProfName)
+					f << "cfn=" << it->first->profName << endl;
+				else
+					f << "cfn=" << it->first << endl;
+				f << "calls=1 1" << endl;
+				f << "1 " << it->second << endl;
+			}
+		}
+	}
+}
+#endif
+
 ABCVm::ABCVm(SystemState* s):m_sys(s),status(CREATED),shuttingdown(false)
 {
 	sem_init(&event_queue_mutex,0,1);
@@ -1195,23 +1180,6 @@ void ABCVm::handleEvent(std::pair<EventDispatcher*, Event*> e)
 				ev->context->exec();
 				break;
 			}
-			case CONSTRUCT_OBJECT:
-			{
-				ConstructObjectEvent* ev=static_cast<ConstructObjectEvent*>(e.second);
-				LOG(LOG_CALLS,_("Building instance traits"));
-				try
-				{
-					ev->_class->handleConstruction(ev->_obj,NULL,0,true);
-				}
-				catch(LightsparkException& e)
-				{
-					//Sync anyway and rethrow
-					ev->sync();
-					throw;
-				}
-				ev->sync();
-				break;
-			}
 			case CHANGE_FRAME:
 			{
 				FrameChangeEvent* ev=static_cast<FrameChangeEvent*>(e.second);
@@ -1219,6 +1187,12 @@ void ABCVm::handleEvent(std::pair<EventDispatcher*, Event*> e)
 				ev->movieClip->state.explicit_FP=true;
 				break;
 			}
+			case CONSTRUCT_FRAME:
+			{
+				ConstructFrameEvent* ev=static_cast<ConstructFrameEvent*>(e.second);
+				ev->frame.construct(ev->parent);
+				break;
+			}
 			default:
 				throw UnsupportedException("Not supported event");
 		}
@@ -1236,7 +1210,7 @@ bool ABCVm::addEvent(EventDispatcher* obj ,Event* ev)
 		return false;
 	//If the event is a synchronization and we are running in the VM context
 	//we should handle it immidiately to avoid deadlock
-	if(isVmThread && (ev->getEventType()==SYNC || ev->getEventType()==CONSTRUCT_OBJECT))
+	if(isVmThread && (ev->getEventType()==SYNC))
 	{
 		assert(obj==NULL);
 		ev->incRef();
@@ -1256,16 +1230,6 @@ bool ABCVm::addEvent(EventDispatcher* obj ,Event* ev)
 
 void ABCVm::buildClassAndInjectBase(const string& s, ASObject* base, ASObject* const* args, const unsigned int argslen, bool isRoot)
 {
-	//It seems to be acceptable for the same base to be binded multiple times,
-	//We refuse to do it, as it's an undocumented behaviour, but we warn the user
-	//I've seen this behaviour only for youtube
-	DictionaryTag* t=dynamic_cast<DictionaryTag*>(base);
-	if(t && t->bindedTo && !isRoot)
-	{
-		LOG(LOG_NOT_IMPLEMENTED,_("Multiple binding on ") << s << _(". Not binding"));
-		return;
-	}
-
 	LOG(LOG_CALLS,_("Setting class name to ") << s);
 	ASObject* target;
 	ASObject* derived_class=Global->getVariableByString(s,target);
@@ -1309,10 +1273,17 @@ void ABCVm::buildClassAndInjectBase(const string& s, ASObject* base, ASObject* c
 	}
 	else
 	{
+		DictionaryTag* t=dynamic_cast<DictionaryTag*>(base);
 		//If this is not a root movie clip, then the base has to be a DictionaryTag
-		assert_and_throw(t);
+		assert(t);
+
+		//It seems to be acceptable for the same base to be binded multiple times.
+		//In such cases the first binding is bidirectional (instances created using PlaceObject
+		//use the binded class and instances created using 'new' use the binded tag). Any other
+		//bindings will be unidirectional (only instances created using new will use the binded tag)
+		if(t->bindedTo==NULL)
+			t->bindedTo=derived_class_tmp;
 
-		t->bindedTo=derived_class_tmp;
 		derived_class_tmp->bindToTag(t);
 	}
 }
@@ -1807,18 +1778,19 @@ void ABCContext::buildTrait(ASObject* obj, const traits_info* t, bool isBorrowed
 			LOG(LOG_CALLS,_("Getter trait: ") << mname << _(" #") << t->method);
 			//syntetize method and create a new LLVM function object
 			method_info* m=&methods[t->method];
-			if(m->name==0)
-			{
-				//Use this name as the method name
-				m->name=t->name;
-			}
 			SyntheticFunction* f=Class<IFunction>::getSyntheticFunction(m);
 
 			//We have to override if there is a method with the same name,
 			//even if the namespace are different, if both are protected
 			assert_and_throw(obj->getObjectType()==T_CLASS);
 			Class_inherit* prot=static_cast<Class_inherit*>(obj);
-			assert(prot);
+#ifdef PROFILING_SUPPORT
+			if(!m->validProfName)
+			{
+				m->profName=prot->class_name.name+"::"+mname.qualifiedString();
+				m->validProfName=true;
+			}
+#endif
 			if(t->kind&0x20 && prot->use_protected && mname.ns[0]==prot->protected_ns)
 			{
 				//Walk the super chain and find variables to override
@@ -1851,11 +1823,6 @@ void ABCContext::buildTrait(ASObject* obj, const traits_info* t, bool isBorrowed
 			LOG(LOG_CALLS,_("Setter trait: ") << mname << _(" #") << t->method);
 			//syntetize method and create a new LLVM function object
 			method_info* m=&methods[t->method];
-			if(m->name==0)
-			{
-				//Use this name as the method name
-				m->name=t->name;
-			}
 
 			IFunction* f=Class<IFunction>::getSyntheticFunction(m);
 
@@ -1863,7 +1830,13 @@ void ABCContext::buildTrait(ASObject* obj, const traits_info* t, bool isBorrowed
 			//even if the namespace are different, if both are protected
 			assert_and_throw(obj->getObjectType()==T_CLASS);
 			Class_base* prot=static_cast<Class_base*>(obj);
-			assert(prot);
+#ifdef PROFILING_SUPPORT
+			if(!m->validProfName)
+			{
+				m->profName=prot->class_name.name+"::"+mname.qualifiedString();
+				m->validProfName=true;
+			}
+#endif
 			if(t->kind&0x20 && prot->use_protected && mname.ns[0]==prot->protected_ns)
 			{
 				//Walk the super chain and find variables to override
@@ -1896,11 +1869,6 @@ void ABCContext::buildTrait(ASObject* obj, const traits_info* t, bool isBorrowed
 			LOG(LOG_CALLS,_("Method trait: ") << mname << _(" #") << t->method);
 			//syntetize method and create a new LLVM function object
 			method_info* m=&methods[t->method];
-			if(m->name==0)
-			{
-				//Use this name as the method name
-				m->name=t->name;
-			}
 			SyntheticFunction* f=Class<IFunction>::getSyntheticFunction(m);
 
 			if(obj->getObjectType()==T_CLASS)
@@ -1910,7 +1878,13 @@ void ABCContext::buildTrait(ASObject* obj, const traits_info* t, bool isBorrowed
 				//We have to override if there is a method with the same name,
 				//even if the namespace are different, if both are protected
 				Class_inherit* prot=static_cast<Class_inherit*>(obj);
-				assert(prot);
+#ifdef PROFILING_SUPPORT
+				if(!m->validProfName)
+				{
+					m->profName=prot->class_name.name+"::"+mname.qualifiedString();
+					m->validProfName=true;
+				}
+#endif
 				if(t->kind&0x20 && prot->use_protected && mname.ns[0]==prot->protected_ns)
 				{
 					//Walk the super chain and find variables to override
@@ -1938,6 +1912,13 @@ void ABCContext::buildTrait(ASObject* obj, const traits_info* t, bool isBorrowed
 			{
 				//Script method
 				f->addToScope(obj);
+#ifdef PROFILING_SUPPORT
+				if(!m->validProfName)
+				{
+					m->profName=mname.qualifiedString();
+					m->validProfName=true;
+				}
+#endif
 			}
 			else //TODO: transform in a simple assert
 				assert_and_throw(obj->getObjectType()==T_CLASS || deferred_initialization);
