@@ -351,6 +351,8 @@ public:
 		return val ? 1.0 : 0.0;
 	}
 	static void buildTraits(ASObject* o){};
+	static void sinit(Class_base*);
+	ASFUNCTION(_toString);
 	ASFUNCTION(generator);
 };
 
@@ -420,6 +422,7 @@ public:
 class ASQName: public ASObject
 {
 friend class ABCContext;
+friend class Namespace;
 CLASSBUILDABLE(ASQName);
 private:
 	tiny_string uri;
@@ -428,6 +431,8 @@ private:
 public:
 	static void sinit(Class_base*);
 	ASFUNCTION(_constructor);
+	ASFUNCTION(_getURI);
+	ASFUNCTION(_getLocalName);
 };
 
 class Namespace: public ASObject
@@ -437,12 +442,17 @@ friend class ABCContext;
 CLASSBUILDABLE(Namespace);
 private:
 	tiny_string uri;
+	tiny_string prefix;
 	Namespace(){type=T_NAMESPACE;}
 	Namespace(const tiny_string& _uri):uri(_uri){type=T_NAMESPACE;}
 public:
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
 	ASFUNCTION(_constructor);
+	ASFUNCTION(_getURI);
+	ASFUNCTION(_setURI);
+	ASFUNCTION(_getPrefix);
+	ASFUNCTION(_setPrefix);
 };
 
 enum DATA_TYPE {DATA_OBJECT=0,DATA_INT};
@@ -683,11 +693,16 @@ public:
 	ASFUNCTION(appendChild);
 	ASFUNCTION(localName);
 	ASFUNCTION(generator);
+	ASFUNCTION(_hasSimpleContent);
+	ASFUNCTION(_hasComplexContent);
 	static void buildTraits(ASObject* o){};
 	static void sinit(Class_base* c);
 	void getDescendantsByQName(const tiny_string& name, const tiny_string& ns, std::vector<XML*>& ret);
 	ASObject* getVariableByMultiname(const multiname& name, bool skip_impl, ASObject* base=NULL);
 	tiny_string toString(bool debugMsg=false);
+	bool hasSimpleContent() const;
+	bool hasComplexContent() const;
+        xmlElementType getNodeKind() const;
 };
 
 class XMLList: public ASObject
@@ -704,8 +719,12 @@ public:
 	ASFUNCTION(_constructor);
 	ASFUNCTION(_getLength);
 	ASFUNCTION(appendChild);
+	ASFUNCTION(_hasSimpleContent);
+	ASFUNCTION(_hasComplexContent);
 	ASObject* getVariableByMultiname(const multiname& name, bool skip_impl, ASObject* base=NULL);
 	XML* convertToXML() const;
+	bool hasSimpleContent() const;
+	bool hasComplexContent() const;
 };
 
 class Date: public ASObject
@@ -825,6 +844,7 @@ public:
 	ASFUNCTION(_setMessage);
 	ASFUNCTION(_getMessage);
 	ASFUNCTION(_getErrorID);
+	ASFUNCTION(_toString);
 	tiny_string toString(bool debugMsg=false);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
